package com.woolha.jpamulticolumnselect.service;

import com.woolha.jpamulticolumnselect.model.projection.OrderDto;
import com.woolha.jpamulticolumnselect.model.projection.OrderView;
import com.woolha.jpamulticolumnselect.model.projection.OrderWithLinesDto;
import com.woolha.jpamulticolumnselect.model.projection.OrderWithLinesView;
import com.woolha.jpamulticolumnselect.repository.OrderRepository;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

@RequiredArgsConstructor
@Service
public class OrderService {

  private final OrderRepository orderRepository;

  public void fetchWithInterfaceProjection(UUID id) {
    Optional<OrderView> orderViewOptional = this.orderRepository.findOneProjectedById(id);

    if (orderViewOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    OrderView orderView = orderViewOptional.get();
    System.out.println("ID : " + orderView.getId());
    System.out.println("Order number: " + orderView.getOrderNumber());
    System.out.println("Total price: " + orderView.getTotalPrice());
    System.out.println("Created at: " + orderView.getCreatedAt());
  }

  public void fetchWithNestedInterfaceProjection(UUID userId) {
    List<OrderWithLinesView> orders = this.orderRepository.findAllProjectedWithLinesByUserId(userId);

    if (orders.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    for (OrderWithLinesView orderView : orders) {
      System.out.println("ID : " + orderView.getId());
      System.out.println("Order number: " + orderView.getOrderNumber());
      System.out.println("Total price: " + orderView.getTotalPrice());
      System.out.println("Created at: " + orderView.getCreatedAt());

      for (OrderWithLinesView.OrderLineView orderLineView : orderView.getOrderLines()) {
        System.out.println("-------------");
        System.out.println("OrderLine ID: " + orderLineView.getId());
        System.out.println("OrderLine product name: " + orderLineView.getProductName());
        System.out.println("OrderLine quantity: " + orderLineView.getQuantity());
        System.out.println("OrderLine price: " + orderLineView.getPrice());
      }
    }
  }

  public void fetchWithScalarProjection(UUID userId) {
    List<Object[]> orders = this.orderRepository.findAllProjectedByUserId(userId);

    if (orders.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    for (Object[] order : orders) {
      System.out.println("ID : " + order[0]);
      System.out.println("Order number: " + order[1]);
      System.out.println("Total price: " + order[2]);
      System.out.println("Created at: " + order[3]);
    }
  }

  public void fetchWithClassProjection(String orderNumber) {
//    List<OrderDto> orderDtoOptional = this.orderRepository.findOneProjectedByOrderNumber(orderNumber);
    Optional<OrderDto> orderDtoOptional = this.orderRepository.findOneProjectedByOrderNumber(orderNumber);

    if (orderDtoOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    OrderDto orderDto = orderDtoOptional.get();
    System.out.println("ID : " + orderDto.getId());
    System.out.println("Order number: " + orderDto.getOrderNumber());
    System.out.println("Total price: " + orderDto.getTotalPrice());
    System.out.println("Created at: " + orderDto.getCreatedAt());
  }

  public void fetchWithCriteriaQuery(UUID id) {
    Optional<OrderDto> orderDtoOptional = this.orderRepository.findOneProjectedUsingCriteriaQueryById(id);

    if (orderDtoOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    OrderDto orderDto = orderDtoOptional.get();
    System.out.println("ID : " + orderDto.getId());
    System.out.println("Order number: " + orderDto.getOrderNumber());
    System.out.println("Total price: " + orderDto.getTotalPrice());
    System.out.println("Created at: " + orderDto.getCreatedAt());
  }

  public void fetchWithCriteriaQueryWithJoin(UUID id) {
    Optional<OrderWithLinesDto> orderDtoOptional = this.orderRepository.findOneWithLinesProjectedUsingCriteriaQueryById(id);

    if (orderDtoOptional.isEmpty()) {
      System.out.println("Not found");
      return;
    }

    OrderWithLinesDto orderDto = orderDtoOptional.get();
    System.out.println("ID : " + orderDto.getId());
    System.out.println("Order number: " + orderDto.getOrderNumber());
    System.out.println("Total price: " + orderDto.getTotalPrice());
    System.out.println("Created at: " + orderDto.getCreatedAt());

    for (OrderWithLinesDto.OrderLineDto orderLine : orderDto.getOrderLines()) {
      System.out.println("----------------");
      System.out.println("Order line ID: " + orderLine.getId());
      System.out.println("Order line quantity: " + orderLine.getQuantity());
      System.out.println("Order line product name: " + orderLine.getProductName());
      System.out.println("Order line price: " + orderLine.getPrice());
    }
  }
}
